@@ -5,32 +5,50 @@ miniwdl command-line interface
 import sys
 import os
 import platform
-import subprocess
 import tempfile
-import glob
 import json
-import math
 import argcomplete
 import logging
 import urllib
 import asyncio
 import atexit
-import docker
+import textwrap
 from shlex import quote as shellquote
-from datetime import datetime
 from argparse import ArgumentParser, Action, SUPPRESS, RawDescriptionHelpFormatter
 from contextlib import ExitStack
 import importlib_metadata
 from ruamel.yaml import YAML
-from . import *
+from . import (
+    load,
+    runtime,
+    Error,
+    Lint,
+    Value,
+    Type,
+    Expr,
+    Document,
+    Workflow,
+    Task,
+    Env,
+    Decl,
+    Call,
+    Scatter,
+    Conditional,
+    SourcePosition,
+    parse_document,
+    copy_source,
+    values_from_json,
+    values_to_json,
+    read_source_default,
+    ReadSourceResult,
+)
 from ._util import (
-    provision_run_dir,
-    write_values_json,
     VERBOSE_LEVEL,
     NOTICE_LEVEL,
     configure_logger,
     parse_byte_size,
     path_really_within,
+    ANSI,
 )
 from ._util import StructuredLogMessage as _
 
@@ -72,7 +90,8 @@ def main(args=None):
         print_error(exn)
         if args.check_quant and quant_warning:
             print(
-                "* Hint: for compatibility with older existing WDL code, try setting --no-quant-check to relax quantifier validation rules.",
+                "* Hint: for compatibility with older existing WDL code, try setting --no-quant-check to relax "
+                "quantifier validation rules.",
                 file=sys.stderr,
             )
         if args.debug:
@@ -125,7 +144,10 @@ def fill_common(subparser, path=True):
         "--no-quant-check",
         dest="check_quant",
         action="store_false",
-        help="relax static typechecking of optional types, and permit coercion of T to Array[T] (discouraged; for backwards compatibility with older WDL)",
+        help=(
+            "relax static typechecking of optional types, and permit coercion of T to Array[T] (discouraged; for "
+            "backwards compatibility with older WDL)"
+        ),
     )
     if path:
         group.add_argument(
@@ -145,6 +167,7 @@ def fill_common(subparser, path=True):
 def fill_check_subparser(subparsers):
     check_parser = subparsers.add_parser(
         "check",
+        help="Validate a WDL document; show an outline with lint warnings",
         description="Load and typecheck a WDL document, showing an outline with lint warnings.\n\n"
         "Individual lint warnings can be suppressed by a WDL comment containing !WarningName on the\n"
         "same line or the following line, e.g.:\n"
@@ -203,9 +226,10 @@ def check(
         print(os.path.basename(uri1))
         outline(doc, 0, show_called=(doc.workflow is not None), show_all=show_all, shown=shown)
 
-    if shellcheck and Lint._shellcheck_available == False:
+    if shellcheck and Lint._shellcheck_available is False:
         print(
-            "* Suggestion: install shellcheck (www.shellcheck.net) to check task commands. (--no-shellcheck suppresses this message)",
+            "* Suggestion: install shellcheck (www.shellcheck.net) to check task commands. (--no-shellcheck "
+            "suppresses this message)",
             file=sys.stderr,
         )
 
@@ -222,10 +246,10 @@ def outline(obj, level, file=sys.stdout, show_called=True, show_all=False, shown
     def descend(dobj=None, first_descent=first_descent):
         # show lint for the node just prior to first descent beneath it
         if not first_descent and hasattr(obj, "lint"):
-            for (pos, klass, msg, suppressed) in sorted(obj.lint, key=lambda t: t[0]):
+            for (pos, cls, msg, suppressed) in sorted(obj.lint, key=lambda t: t[0]):
                 if show_all or not suppressed:
                     print(
-                        f"{s}    (Ln {pos.line}, Col {pos.column}) {klass}{' (suppressed)' if suppressed else ''}, {msg}",
+                        f"{s}    (Ln {pos.line}, Col {pos.column}) {cls}{' (suppressed)' if suppressed else ''}, {msg}",
                         file=file,
                     )
                     if shown:
@@ -302,13 +326,14 @@ def print_error(exn):
         for exn1 in exn.exceptions:
             print_error(exn1)
     else:
+        if sys.stderr.isatty():
+            sys.stderr.write(ANSI.BHRED)
         if isinstance(getattr(exn, "pos", None), SourcePosition):
-            print(
-                "({} Ln {} Col {}) {}".format(exn.pos.uri, exn.pos.line, exn.pos.column, str(exn)),
-                file=sys.stderr,
-            )
+            print(f"({exn.pos.uri} Ln {exn.pos.line} Col {exn.pos.column}) {exn}", file=sys.stderr)
         else:
             print(str(exn), file=sys.stderr)
+        if sys.stderr.isatty():
+            sys.stderr.write(ANSI.RESET)
         if isinstance(exn, Error.ImportError) and hasattr(exn, "__cause__"):
             print_error(exn.__cause__)
         if isinstance(exn, Error.ValidationError) and exn.source_text:
@@ -352,7 +377,10 @@ async def read_source(uri, path, importer):
 
 def fill_run_subparser(subparsers):
     run_parser = subparsers.add_parser(
-        "run", help="Run workflow/task locally with built-in runtime [beta test]"
+        "run",
+        help="Run workflow/task locally with built-in runtime",
+        description="For details & configuration see:\n"
+        "https://miniwdl.readthedocs.io/en/latest/runner_reference.html",
     )
     run_parser.add_argument("uri", metavar="URI", type=str, help="WDL document filename/URI")
     run_parser.add_argument(
@@ -394,7 +422,10 @@ def fill_run_subparser(subparsers):
         "--dir",
         metavar="DIR",
         dest="run_dir",
-        help="directory under which to create a timestamp-named subdirectory for this run (defaults to current working directory); supply '.' or 'some/dir/.' to instead run in this directory exactly",
+        help=(
+            "directory under which to create a timestamp-named subdirectory for this run (defaults to current "
+            " working directory); supply '.' or 'some/dir/.' to instead run in this directory exactly"
+        ),
     )
     group.add_argument(
         "--error-json",
@@ -420,7 +451,10 @@ def fill_run_subparser(subparsers):
         metavar="FILE",
         type=str,
         default=None,
-        help="configuration file to load (in preference to file named by MINIWDL_CFG environment, or XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)",
+        help=(
+            "configuration file to load (in preference to file named by MINIWDL_CFG environment, or "
+            "XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)"
+        ),
     )
     group.add_argument("-@", metavar="N", dest="max_tasks", type=int, default=None, help=SUPPRESS)
     group.add_argument("--runtime-cpu-max", metavar="N", type=int, default=None, help=SUPPRESS)
@@ -445,7 +479,10 @@ def fill_run_subparser(subparsers):
     group.add_argument(
         "--as-me",
         action="store_true",
-        help="run all containers as the invoking user uid:gid (more secure, but potentially blocks task commands e.g. apt-get)",
+        help=(
+            "run all containers as the invoking user uid:gid (more secure, but potentially blocks task commands e.g. "
+            "apt-get)"
+        ),
     )
     # TODO:
     # way to specify None for an optional value (that has a default)
@@ -499,7 +536,10 @@ def runner(
 
         if os.geteuid() == 0:
             logger.warning(
-                "running as root; non-root users should be able to `miniwdl run` as long as they're in the `docker` group"
+                (
+                    "running as root; non-root users should be able to `miniwdl run` "
+                    "as long as they're in the `docker` group"
+                )
             )
 
         # load configuration & apply command-line overrides
@@ -565,26 +605,35 @@ def runner(
             )
             sys.exit(2)
 
-        # load WDL document
-        doc = load(uri, path or [], check_quant=check_quant, read_source=read_source)
+        try:
+            # load WDL document
+            doc = load(uri, path or [], check_quant=check_quant, read_source=read_source)
 
-        # parse and validate the provided inputs
-        def file_found(fn):
-            return runtime.download.able(cfg, fn) or os.path.isfile(fn)
+            # parse and validate the provided inputs
+            def file_found(fn):
+                return runtime.download.able(cfg, fn) or os.path.isfile(fn)
 
-        eff_root = (
-            cfg["file_io"]["root"] if not cfg["file_io"].get_bool("copy_input_files") else "/"
-        )
+            eff_root = (
+                cfg["file_io"]["root"] if not cfg["file_io"].get_bool("copy_input_files") else "/"
+            )
 
-        target, input_env, input_json = runner_input(
-            doc,
-            inputs,
-            input_file,
-            empty,
-            task=task,
-            file_found=file_found,
-            root=eff_root,  # if copy_input_files is set, then input files need not reside under the configured root
-        )
+            target, input_env, input_json = runner_input(
+                doc,
+                inputs,
+                input_file,
+                empty,
+                task=task,
+                file_found=file_found,
+                root=eff_root,  # if copy_input_files is set, then input files need not reside under the configured root
+            )
+        except Error.InputError as exn:
+            if error_json:
+                print(json.dumps(runtime.error_json(exn), indent=2))
+            die(exn.args[0])
+        except Exception as exn:
+            if error_json:
+                print(json.dumps(runtime.error_json(exn), indent=2))
+            raise
 
         if json_only:
             print(json.dumps(input_json, indent=2))
@@ -626,9 +675,6 @@ def runner(
 
         rerun_sh = f"pushd {shellquote(os.getcwd())} && miniwdl {' '.join(shellquote(t) for t in sys.argv[1:])}; popd"
 
-        # initialize local Docker Swarm
-        runtime.task.SwarmContainer.global_init(cfg, logger)
-
         # run & log any errors
         cleanup.enter_context(runtime._statusbar.enable(set_status))
         cache = cleanup.enter_context(runtime.cache.CallCache(cfg, logger))
@@ -661,7 +707,7 @@ def runner(
                 msg = info["message"]
                 del info["message"]
             logger.error(_(msg, **info))
-            if kwargs["debug"]:
+            if isinstance(exn, AssertionError) or kwargs["debug"]:
                 raise
             sys.exit(exit_status)
         finally:
@@ -739,15 +785,17 @@ def runner_input(
     if task:
         target = next((t for t in doc.tasks if t.name == task), None)
         if not target:
-            die(f"no such task {task} in document")
+            raise Error.InputError(f"no such task {task} in document")
     elif doc.workflow:
         target = doc.workflow
     elif len(doc.tasks) == 1:
         target = doc.tasks[0]
     elif len(doc.tasks) > 1:
-        die("specify --task for WDL document with multiple tasks and no workflow")
+        raise Error.InputError(
+            "specify --task for WDL document with multiple tasks and no workflow"
+        )
     else:
-        die("Empty WDL document")
+        raise Error.InputError("Empty WDL document")
     assert target
 
     # build up an values env of the provided inputs
@@ -761,36 +809,28 @@ def runner_input(
         try:
             decl = available_inputs[empty_name]
         except KeyError:
-            die(
-                "No such input to {}: {}\n{}".format(
-                    target.name, empty_name, runner_input_help(target)
-                )
-            )
+            runner_input_help(target)
+            raise Error.InputError(f"No such input to {target.name}: {empty_name}")
         if not isinstance(decl.type, Type.Array) or decl.type.nonempty:
-            die("Cannot set input {} {} to empty array".format(str(decl.type), decl.name))
+            runner_input_help(target)
+            raise Error.InputError(f"Cannot set input {str(decl.type)} {decl.name} to empty array")
         input_env = input_env.bind(empty_name, Value.Array(decl.type.item_type, []), decl)
 
     # add in command-line inputs
     for one_input in inputs:
-        if not one_input or not one_input[0].isalpha():
-            # let user just see runner_input_help
-            die(
-                f"{target.name} ({target.pos.uri})\n{'-'*(len(target.name)+len(target.pos.uri)+3)}\n{runner_input_help(target)}"
-            )
-
         # parse [namespace], name, and value
         buf = one_input.split("=", 1)
-        if len(buf) != 2 or not buf[0]:
-            die("Invalid input name=value pair: " + one_input)
+        if not one_input or not one_input[0].isalpha() or len(buf) != 2 or not buf[0]:
+            runner_input_help(target)
+            raise Error.InputError("Invalid input name=value pair: " + one_input)
         name, s_value = buf
 
         # find corresponding input declaration
         try:
             decl = available_inputs[name]
         except KeyError:
-            die(
-                "No such input to {}: {}\n{}".format(target.name, buf[0], runner_input_help(target))
-            )
+            runner_input_help(target)
+            raise Error.InputError(f"No such input to {target.name}: {buf[0]}")
 
         # create a Value based on the expected type
         v = runner_input_value(s_value, decl.type, file_found, root)
@@ -805,7 +845,8 @@ def runner_input(
                 assert isinstance(existing, Value.Array) and v.type.coerces(existing.type)
                 existing.value.extend(v.value)
             else:
-                die("non-array input {} duplicated\n{}".format(buf[0], runner_input_help(target)))
+                runner_input_help(target)
+                raise Error.InputError(f"non-array input {buf[0]} duplicated")
         else:
             input_env = input_env.bind(name, v, decl)
 
@@ -813,10 +854,9 @@ def runner_input(
     if check_required:
         missing_inputs = values_to_json(target.required_inputs.subtract(input_env))
         if missing_inputs:
-            die(
-                "missing required inputs for {}: {}\n{}".format(
-                    target.name, ", ".join(missing_inputs.keys()), runner_input_help(target)
-                )
+            runner_input_help(target)
+            raise Error.InputError(
+                f"missing required inputs for {target.name}: {', '.join(missing_inputs.keys())}"
             )
 
     # make a pass over the Env to create a dict for Cromwell-style input JSON
@@ -851,8 +891,7 @@ def runner_input_json_file(available_inputs, namespace, input_file, root):
         try:
             ans = values_from_json(input_json, available_inputs, namespace=namespace)
         except Error.InputError as exn:
-            die("check JSON input; " + str(exn))
-            sys.exit(2)
+            raise Error.InputError("check JSON input; " + exn.args[0])
 
         # join relative file paths to the cwd
 
@@ -862,10 +901,10 @@ def runner_input_json_file(available_inputs, namespace, input_file, root):
                     if not os.path.isabs(v.value):
                         v.value = os.path.normpath(os.path.join(os.getcwd(), v.value))
                     if not path_really_within(v.value, root):
-                        die(
-                            f"all input files must be located within the configured `file_io.root' directory `{root}' unlike `{v.value}'"
+                        raise Error.InputError(
+                            f"all input files must be located within the configured `file_io.root' directory `{root}' "
+                            f"unlike `{v.value}'"
                         )
-                        sys.exit(2)
             for ch in v.children:
                 absolutify_files(ch)
             return v
@@ -876,13 +915,21 @@ def runner_input_json_file(available_inputs, namespace, input_file, root):
 
 
 def runner_input_help(target):
-    # TODO: get help message from parameter_meta
-    # TODO: show default values of optionals
-    ans = []
+    def bold(line):
+        if sys.stderr.isatty():
+            return f"{ANSI.BOLD}{line}{ANSI.RESET}"
+        return line
+
+    ans = [
+        "",
+        bold(f"{target.name} ({target.pos.uri})"),
+        bold(f"{'-'*(len(target.name)+len(target.pos.uri)+3)}"),
+    ]
     required_inputs = target.required_inputs
-    ans.append("\nrequired inputs:")
+    ans.append(bold("\nrequired inputs:"))
     for b in required_inputs:
-        ans.append("  {} {}".format(str(b.value.type), b.name))
+        ans.append(bold(f"  {str(b.value.type)} {b.name}"))
+        add_wrapped_parameter_meta(target, b.name, ans)
     optional_inputs = target.available_inputs.subtract(target.required_inputs)
     if target.inputs is None:
         # if the target doesn't have an input{} section (pre WDL 1.0), exclude
@@ -891,13 +938,19 @@ def runner_input_help(target):
             lambda b: b.value.expr is None or is_constant_expr(b.value.expr)
         )
     if optional_inputs:
-        ans.append("\noptional inputs:")
+        ans.append(bold("\noptional inputs:"))
         for b in optional_inputs:
-            ans.append("  {} {}".format(str(b.value.type), b.name))
-    ans.append("\noutputs:")
+            d = bold(f"  {str(b.value.type)} {b.name}")
+            if b.value.expr:
+                ans.append(f"{d} = {b.value.expr}")
+            else:
+                ans.append(d)
+            add_wrapped_parameter_meta(target, b.name, ans)
+    ans.append(bold("\noutputs:"))
     for b in target.effective_outputs:
-        ans.append("  {} {}".format(str(b.value), b.name))
-    return "\n".join(ans)
+        ans.append(bold(f"  {str(b.value)} {b.name}"))
+    for line in ans:
+        print(line, file=sys.stderr)
 
 
 def is_constant_expr(expr):
@@ -916,6 +969,18 @@ def is_constant_expr(expr):
     return False
 
 
+def add_wrapped_parameter_meta(target, input_name, output_list):
+    ans = ""
+    if input_name in target.parameter_meta:
+        entry = target.parameter_meta[input_name]
+        if isinstance(entry, str):
+            ans = entry
+        elif isinstance(entry, dict) and isinstance(entry.get("help", None), str):
+            ans = entry["help"]
+    if ans:
+        output_list.extend((" " * 4 + line) for line in textwrap.wrap(ans, 96))
+
+
 def runner_input_value(s_value, ty, file_found, root):
     """
     Given an input value from the command line (right-hand side of =) and the
@@ -928,18 +993,21 @@ def runner_input_value(s_value, ty, file_found, root):
         if os.path.isfile(fn):
             fn = os.path.abspath(fn)
             if not path_really_within(fn, root):
-                die(
-                    f"all input files must be located within the configured `file_io.root' directory `{root}' unlike `{fn}'"
+                raise Error.InputError(
+                    f"all input files must be located within the configured `file_io.root' directory `{root}' "
+                    f"unlike `{fn}'"
                 )
         elif not (file_found and file_found(fn)):  # maybe URI
-            die("File not found: " + fn)
+            raise Error.InputError("File not found: " + fn)
         return Value.File(fn)
     if isinstance(ty, Type.Boolean):
         if s_value == "true":
             return Value.Boolean(True)
         if s_value == "false":
             return Value.Boolean(False)
-        die("Boolean input should be true or false instead of {}".format(s_value))
+        raise Error.InputError(
+            "Boolean input should be true or false instead of `{}'".format(s_value)
+        )
     if isinstance(ty, Type.Int):
         return Value.Int(int(s_value))
     if isinstance(ty, Type.Float):
@@ -951,7 +1019,7 @@ def runner_input_value(s_value, ty, file_found, root):
         return Value.Array(
             ty.item_type, [runner_input_value(s_value, ty.item_type, file_found, root)]
         )
-    return die(
+    raise Error.InputError(
         "No command-line support yet for inputs of type {}; workaround: specify in JSON file with --input".format(
             str(ty)
         )
@@ -960,8 +1028,7 @@ def runner_input_value(s_value, ty, file_found, root):
 
 def fill_run_self_test_subparser(subparsers):
     run_parser = subparsers.add_parser(
-        "run_self_test",
-        help="Run a trivial workflow to smoke-test installation, docker permission, etc.",
+        "run_self_test", help="Run a short built-in workflow to test system configuration",
     )
     run_parser.add_argument(
         "--dir",
@@ -974,7 +1041,10 @@ def fill_run_self_test_subparser(subparsers):
         metavar="FILE",
         type=str,
         default=None,
-        help="configuration file to load (in preference to file named by MINIWDL_CFG environment, or XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)",
+        help=(
+            "configuration file to load (in preference to file named by MINIWDL_CFG environment, "
+            "or XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)"
+        ),
     )
     run_parser.add_argument("--log-json", action="store_true", help="write all logs in JSON")
     run_parser.add_argument(
@@ -989,7 +1059,7 @@ def run_self_test(**kwargs):
         dn = tempfile.mkdtemp(prefix="miniwdl_run_self_test_")
     with open(os.path.join(dn, "test.wdl"), "w") as outfile:
         outfile.write(
-            """
+            r"""
             version 1.0
             workflow hello_caller {
                 input {
@@ -1056,7 +1126,8 @@ def run_self_test(**kwargs):
                 lambda: print(
                     "* Hint: ensure Docker is installed & running"
                     + (
-                        ", and user has permission to control it per https://docs.docker.com/install/linux/linux-postinstall/#manage-docker-as-a-non-root-user"
+                        ", and user has permission to control it per "
+                        "https://docs.docker.com/install/linux/linux-postinstall/#manage-docker-as-a-non-root-user"
                         if platform.system() != "Darwin"
                         else "; and on macOS override the environment variable TMPDIR=/tmp/"
                     ),
@@ -1068,7 +1139,8 @@ def run_self_test(**kwargs):
     print("miniwdl run_self_test OK", file=sys.stderr)
     if os.geteuid() == 0:
         print(
-            "* Hint: non-root users should be able to run miniwdl if they have permission to control Docker per https://docs.docker.com/install/linux/linux-postinstall/#manage-docker-as-a-non-root-user",
+            "* Hint: non-root users should be able to run miniwdl if they have permission to control Docker per "
+            "https://docs.docker.com/install/linux/linux-postinstall/#manage-docker-as-a-non-root-user",
             file=sys.stderr,
         )
 
@@ -1076,8 +1148,9 @@ def run_self_test(**kwargs):
 def fill_localize_subparser(subparsers):
     localize_parser = subparsers.add_parser(
         "localize",
-        help="Download URI input files to local cache",
-        description=f"Prime the local file download cache with URI File inputs found in Cromwell-style input JSON",
+        help="Download URI input Files to local cache for use in subsequent runs",
+        description="Prime the local file download cache with URI File inputs found in Cromwell-style input JSON. "
+        "This is only needed if it's useful to perform downloads in advance rather than on next run start.",
     )
     localize_parser.add_argument(
         "wdlfile",
@@ -1116,7 +1189,10 @@ def fill_localize_subparser(subparsers):
         metavar="FILE",
         type=str,
         default=None,
-        help="configuration file to load (in preference to file named by MINIWDL_CFG environment, or XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)",
+        help=(
+            "configuration file to load (in preference to file named by MINIWDL_CFG environment, "
+            "or XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)"
+        ),
     )
     return localize_parser
 
@@ -1167,9 +1243,12 @@ def localize(
             def file_found(fn):
                 return runtime.download.able(cfg, fn) or os.path.isfile(fn)
 
-            target, input_env, input_json = runner_input(
-                doc, [], infile, [], task=task, check_required=False, file_found=file_found
-            )
+            try:
+                target, input_env, input_json = runner_input(
+                    doc, [], infile, [], task=task, check_required=False, file_found=file_found
+                )
+            except Error.InputError as exn:
+                die(exn.args[0])
 
             # scan for Files that appear to be downloadable URIs
             def scan(x):
@@ -1183,13 +1262,15 @@ def localize(
 
         if not uri:
             logger.warning(
-                "nothing to do; if inputs use special URI schemes, make sure necessary downloader plugin(s) are installed and enabled"
+                "nothing to do; if inputs use special URI schemes, make sure necessary downloader plugin(s) are "
+                "installed and enabled"
             )
             sys.exit(0)
 
         if not cache_cfg.get_bool("put"):
             logger.error(
-                'configuration section "download_cache", option "put" (env MINIWDL__DOWNLOAD_CACHE__PUT) must be true for this operation to be effective'
+                'configuration section "download_cache", option "put" (env MINIWDL__DOWNLOAD_CACHE__PUT) must be true '
+                "for this operation to be effective"
             )
             sys.exit(2)
 
@@ -1214,9 +1295,6 @@ def localize(
             sys.exit(0)
         logger.notice(_("starting downloads", uri=uri))
 
-        # initialize Docker Swarm
-        runtime.task.SwarmContainer.global_init(cfg, logger)
-
         # cheesy trick: provide the list of URIs as File inputs to a dummy workflow, causing the
         # runtime to download & cache them
         localizer_wdl = """
@@ -1245,10 +1323,15 @@ def localize(
         )
         if not original_get:
             logger.warning(
-                """future runs won't use the cache unless configuration section "download_cache", key "get" (env MINIWDL__DOWNLOAD_CACHE__GET) is set to true"""
+                """future runs won't use the cache unless configuration section "download_cache", key "get" """
+                """(env MINIWDL__DOWNLOAD_CACHE__GET) is set to true"""
             )
 
 
 def die(msg, status=2):
-    print("\n" + msg + "\n", file=sys.stderr)
+    msg = "\n".join(textwrap.wrap(msg, 100))
+    if sys.stderr.isatty():
+        print(f"\n{ANSI.BHRED}{msg}{ANSI.RESET}\n", file=sys.stderr)
+    else:
+        print(f"\n{msg}\n", file=sys.stderr)
     sys.exit(status)
