@@ -61,11 +61,11 @@ def _downloader(
     return getattr(cfg, "_downloaders").get(scheme, None)
 
 
-def able(cfg: config.Loader, uri: str) -> bool:
+def able(cfg: config.Loader, uri: Optional[str]) -> bool:
     """
     Returns True if uri appears to be a URI we know how to download
     """
-    return _downloader(cfg, uri) is not None
+    return bool(uri and _downloader(cfg, uri) is not None)
 
 
 def run(cfg: config.Loader, logger: logging.Logger, uri: str, **kwargs) -> str:
@@ -127,10 +127,9 @@ def run_cached(
     cached = cache.get_download(uri, logger=logger)
     if cached:
         return True, cached
-    if not cfg["download_cache"].get_bool("put") or not cache.download_path(uri):
-        return False, run(cfg, logger, uri, run_dir=run_dir, **kwargs)
-    # run the download within the cache directory
-    run_dir = os.path.join(cfg["download_cache"]["dir"], "ops")
+    if cache.download_cacheable(uri):
+        # run the download within the cache directory
+        run_dir = os.path.join(cfg["download_cache"]["dir"], "ops")
     filename = run(cfg, logger, uri, run_dir=run_dir, **kwargs)
     return False, cache.put_download(uri, os.path.realpath(filename), logger=logger)
 
@@ -228,15 +227,16 @@ def awscli_downloader(
                 if [ -n "~{aws_credentials}" ]; then
                     source "~{aws_credentials}"
                 fi
-                args=""
-                if ! aws sts get-caller-identity >&2 ; then
-                    # no credentials or instance role; add --no-sign-request to allow requests for
-                    # PUBLIC objects to proceed.
-                    args="--no-sign-request"
-                fi
+                set -x
                 mkdir __out
-                cd __out
-                aws s3 cp $args "~{uri}" .
+                if ! aws s3 cp "~{uri}" __out/ ; then
+                    # Retry with --no-sign-request in case the object is public. Without this flag,
+                    # the previous invocation could have failed either because (i) no AWS
+                    # credentials are available or (ii) the IAM policy restricts accessible S3
+                    # buckets regardless of whether the desired object is public.
+                    rm -f __out/*
+                    aws s3 cp --no-sign-request "~{uri}" __out/
+                fi
             >>>
 
             output {
