@@ -15,7 +15,7 @@ import re
 from typing import Tuple, List, Dict, Optional, Callable, Iterable, Set, Any, Union
 from contextlib import ExitStack
 
-from .. import Error, Type, Env, Value, StdLib, Tree, _util
+from .. import Error, Type, Env, Value, StdLib, Tree, Expr, _util
 from .._util import (
     write_atomic,
     write_values_json,
@@ -63,7 +63,6 @@ def run_local_task(
     logger_prefix = (logger_prefix or ["wdl"]) + ["t:" + run_id]
     logger = logging.getLogger(".".join(logger_prefix))
     with ExitStack() as cleanup:
-        cleanup.enter_context(_statusbar.task_slotted())
         terminating = cleanup.enter_context(TerminationSignalFlag(logger))
         if terminating():
             raise Terminated(quiet=True)
@@ -88,9 +87,9 @@ def run_local_task(
                 line=task.pos.line,
                 column=task.pos.column,
                 dir=run_dir,
+                thread=threading.get_ident(),
             )
         )
-        logger.info(_("thread", ident=threading.get_ident()))
         write_values_json(inputs, os.path.join(run_dir, "inputs.json"))
 
         if not _run_id_stack:
@@ -146,7 +145,12 @@ def run_local_task(
 
                 # download input files, if needed
                 posix_inputs = _download_input_files(
-                    cfg, logger, logger_prefix, run_dir, inputs, cache
+                    cfg,
+                    logger,
+                    logger_prefix,
+                    run_dir,
+                    _add_downloadable_default_files(cfg, task.available_inputs, inputs),
+                    cache,
                 )
 
                 # create TaskContainer according to configuration
@@ -172,6 +176,7 @@ def run_local_task(
                 command, container = (recv[k] for k in ("command", "container"))
 
                 # start container & run command (and retry if needed)
+                cleanup.enter_context(_statusbar.task_slotted())
                 _try_task(cfg, logger, container, command, terminating)
 
                 # evaluate output declarations
@@ -278,6 +283,30 @@ def _download_input_files(
     return ans
 
 
+def _add_downloadable_default_files(
+    cfg: config.Loader, available_inputs: Env.Bindings[Tree.Decl], inputs: Env.Bindings[Value.Base],
+) -> Env.Bindings[Value.Base]:
+    """
+    Helper for File URI downloading: look for available File inputs that default to a string
+    constant appearing to be a downloadable URI. For each one, add the default binding to the
+    user-supplied inputs (if not already overridden in them).
+
+    This is to trigger download of the default URIs even though we otherwise don't evaluate input
+    declarations until after processing downloads.
+    """
+    ans = inputs
+    for b in available_inputs:
+        if (
+            isinstance(b.value.type, Type.File)
+            and b.name not in ans
+            and isinstance(b.value.expr, Expr.String)
+        ):
+            maybe_uri = b.value.expr.literal
+            if maybe_uri and downloadable(cfg, maybe_uri.value):
+                ans = ans.bind(b.name, Value.File(maybe_uri.value, b.value.expr))
+    return ans
+
+
 def _eval_task_inputs(
     logger: logging.Logger,
     task: Tree.Task,
